@@ -63,3 +63,9 @@ setClass("lmmSeq", slots = list(
   errors = "character_or_list",
   vars = "list"
 ))
+
+
+ignore_unused_imports <- function() {
+  kableExtra::kable_styling()
+  emmeans::emmeans()
+}
\ No newline at end of file
